
DROP TYPE tJnlPacketInv CASCADE;
CREATE TYPE tJnlPacketInv AS (
	accno	VARCHAR,
	accamt	NUMERIC(19,4),
	deptid	VARCHAR,
	prjid	VARCHAR
);


CREATE OR REPLACE FUNCTION spJnlPacketInv(
	_SaleNo	VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cData	REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT accno, deptid, prjid, SUM(accamt) 
		FROM spJnlPacketInv(_SaleNo, 'SALE') 
		GROUP BY accno, deptid, prjid;
	LOOP
		FETCH cData INTO _retval.accno, _retval.deptid, _retval.prjid, _retval.accamt;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spJnlPacketInv(
	_SaleNo	VARCHAR,
	_Type	VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cData	REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT accno, deptid, prjid, SUM(accamt) 
		FROM spJnlPacketInv(_SaleNo, _Type, FALSE) 
		GROUP BY accno, deptid, prjid;
	LOOP
		FETCH cData INTO _retval.accno, _retval.deptid, _retval.prjid, _retval.accamt;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv ( 
	_SaleNo		VARCHAR,
	_Type		VARCHAR,
	_CheckDeli	BOOLEAN
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cSale	REFCURSOR;
	cData	REFCURSOR;

	_WhID	VARCHAR;
	_ItemID	VARCHAR;
	_PID	VARCHAR;
	_Qty	NUMERIC(19,4);
	_Unit	VARCHAR;
	_DeptID	VARCHAR;
	_PrjID	VARCHAR;
BEGIN

	IF (_Type='SALE') THEN
		OPEN cSale FOR
			SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, d.deptid, d.prjid 
			FROM saled d JOIN item i ON d.itemid=i.itemid 
			WHERE i.itemtype='PACK' AND saleno=_SaleNo
			AND ((_CheckDeli=FALSE AND (d.delino IS NULL OR d.delino='')) 
				OR (_CheckDeli=TRUE AND d.delino IS NOT NULL));
		LOOP
			FETCH cSale INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cSale;
	ELSIF (_Type='BNS') THEN
		OPEN cSale FOR
			SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, NULL, NULL 
			FROM salebns d JOIN item i ON d.itemid=i.itemid 
			WHERE i.itemtype='PACK' AND saleno=_SaleNo;
		LOOP
			FETCH cSale INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cSale;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv_Deli ( 
	_DeliNo		deli.delino%TYPE
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cDeli	REFCURSOR;
	cData	REFCURSOR;

	_WhID	wh.whid%TYPE;
	_ItemID	item.itemid%TYPE;
	_PID	VARCHAR;
	_Qty	delid.qty%TYPE;
	_Unit	delid.unit%TYPE;
	_DeptID	dept.deptid%TYPE;
	_PrjID	prj.prjid%TYPE;
BEGIN
	OPEN cDeli FOR
		SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, d.deptid, d.prjid 
		FROM delid d 
		JOIN item i ON d.itemid=i.itemid 
		WHERE i.itemtype='PACK' AND delino=_DeliNo;
	LOOP
		FETCH cDeli INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		OPEN cData FOR
			SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
		LOOP
			FETCH cData INTO _retval;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cData;
	END LOOP;
	CLOSE cDeli;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv (
	__WhID		VARCHAR,
	__ItemID	VARCHAR,
	__PID		VARCHAR,
	__Qty		NUMERIC(19,4),
	__Unit		VARCHAR,
	__DeptID	VARCHAR,
	__PrjID		VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval		tJnlPacketInv%ROWTYPE;

	_AccNo		VARCHAR;
	_AccAmt		NUMERIC(19,4);

	_BomID		VARCHAR;
	_BomQty		NUMERIC(19,4);
	_BomUnit	VARCHAR;

	_ItemID		VARCHAR;
	_Qty		NUMERIC(19,4);
	_Unit		VARCHAR;
	_PID		VARCHAR;
	
	_BomMultiplier	NUMERIC(19,4);

	cBom		REFCURSOR;
	cData		REFCURSOR;

	_Round		INT;
BEGIN
	_Round:=COALESCE(fReg('ROUND'), '0');

	SELECT INTO _BomID, _BomQty, _BomUnit bomid, qty, unit FROM bom WHERE itemid=__ItemID AND pid=COALESCE(__PID, '') AND active=TRUE;
	IF FOUND THEN
		_BomQty:=fQtyZToQty1(__ItemID, _BomQty, _BomUnit);
		_BomMultiplier:=fQtyZtoQty1(__ItemID, __Qty, __Unit)/_BomQty;

		OPEN cBom FOR
			SELECT d.itemid, d.qty, d.unit, COALESCE(d.pid, ''), i.accinv 
			FROM bomd d JOIN item i ON d.itemid=i.itemid  
			WHERE bomid=_BoMID;
		LOOP
			FETCH cBom INTO _ItemID, _Qty, _Unit, _PID, _AccNo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _BomID bomid FROM bom WHERE itemid=_ItemID AND pid=_PID AND active=TRUE;
			IF FOUND THEN
				OPEN cData FOR
					SELECT * FROM spJnlPacketInv(__WhID, _ItemID, _PID, _Qty*_BomMultiplier, _Unit, __DeptID, __PrjID);
				LOOP
					FETCH cData INTO _retval;
					EXIT WHEN NOT FOUND;

					RETURN NEXT _retval;
				END LOOP;
				CLOSE cData;
			ELSE
				_AccAmt:=fUnitCost(_ItemID, __WhID, _PID, _Qty*_BomMultiplier, _Unit);
				_retval.accno:=_AccNo;
				_retval.accamt:=_AccAmt;
				_retval.deptid:=__DeptID;
				_retval.prjid:=__PrjID;

				RETURN NEXT _retval;
			END IF;
		END LOOP;
		CLOSE cBom;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


